#!/bin/bash

#set -x

get_num_cpu(){
    about=1
    #
    test "$(uname -m)" == "sparc64" && local about=$(cat /proc/cpuinfo | grep CPU | wc -l)
    test "$(uname -m)" != "sparc64" && local about=$(cat /proc/cpuinfo | grep processor | wc -l)
    #
    echo $about
}

get_order_proc(){
    about=""
    #
    if test "$(uname -m)" == "sparc64";then
        about="$(cat /proc/cpuinfo \
            | grep online \
            | awk -F':' '{print $1}' \
            | sed 's/CPU//g' \
            | sort -nr \
            | head -$num_cpu)"
    fi
    #
    if test "$(uname -m)" != "sparc64";then
        about="$(cat /proc/cpuinfo \
            | grep processor \
            | awk -F':' '{print $2}' \
            | sed 's/ //g' \
            | sort -nr \
            | head -$num_cpu)"
    fi
    #
    echo "$about"
}

huge_init(){
    #
    local huge_mnt='/mnt/huge'
    # set
    echo 512 > /proc/sys/vm/nr_hugepages
    echo never > /sys/kernel/mm/transparent_hugepage/enabled
    #
    if ! mount | grep -q hugetlbfs; then
        #
        test -d $huge_mnt || mkdir -p $huge_mnt
        #
       	mount none $huge_mnt -t hugetlbfs &>/dev/null
    fi
}

# save cur huge
nr_hugepages=$(cat /proc/sys/vm/nr_hugepages)
transparent_hugepage=$(cat /sys/kernel/mm/transparent_hugepage/enabled | \
                       cut -d ']' -f1 | \
                       cut -d '[' -f2)
# utils
awk="/usr/bin/gawk"
dprof="/opt/mcst/bin/dprof"
dprof2ptrace="/opt/mcst/bin/dprof2ptrace.sh"
ldis="/opt/mcst/bin/ldis"
huge="hugectl"

#flag
el_util_f=1

# dirs
root=$(pwd)
bin_dir="$root/bin"
log_dir="$root/log"

# base value
platform_spec="$bin_dir/collect.spec"
platform=$(uname -m)
num_cpu=1

[[ $1 =~ ^-?[0-9]+$ ]] && num_cpu=$1 || num_cpu=$(get_num_cpu)

# result
num_cpu_str="${num_cpu}_threads"
[[ "$num_cpu" == "1" ]] && num_cpu_str='SingleThread'
[[ "$num_cpu" == "$(get_num_cpu)" ]] && num_cpu_str='MultiThread'
#
result="$root/peak_test.$(uname -p).$num_cpu_str.result"

# init

! test -d "$bin_dir" && echo "\
Test need compile!
Use:
    $ make" && exit 1

# clear
rm -rf $log_dir
mkdir -p $log_dir

# Run
echo "Запущен пакет Производительность: num_thread = $num_cpu; platform = $platform"

# To dir
cd $bin_dir

# clear spec value
dprof_spec=
oplit_spec=
alu_spec=
fp_spec=
run_spec=
huge_spec=
# sparc operation count in test
op_cnt_32= # float-32
op_cnt_64= # float-64
#
huge_run=

# get spec value
if test -f $platform_spec;then
    echo "Source spec file: $platform_spec"
    source $platform_spec || exit 1
fi

#if ! which proc_run &>/dev/null;then
#    el_util_f=0
#fi

if [ -n "$huge_spec" ];then
    if which $huge &>/dev/null;then
        #
        huge_init
        #
        huge_run="$huge $huge_spec"
    else
       echo "WARNING: $huge не найден; Результаты могут быть ниже ожидаемых" >&2
    fi
fi

echo "Check huge use: $huge_run"
echo "Check priotity use: $el_util_f"

# Run
for puse in *b_puse; do
    # TS start
    tunit_start_point=$(date +"%d.%m.%YT%H:%M:%S")

    #
    # Loop var
    #
    f_base="$log_dir/$puse"
    f_base_time="$f_base.time"
    #
    f_result="$f_base.raw.result"
    f_ts_point="$f_base.ts.point"

    if [[ "$platform" == "e2k" ]];then
        #
        # Get trace
        #
        f_IP_exec_raw="$f_base.raw.IP_exec"
        f_IP_exec="$f_base.IP_exec"
        #f_ptrace="$f_base.ptrace"
        f_stat="$f_base.stat"
        #
        f_ptrace="$puse.ptrace"
        #
        events=0
        number=`grep $puse control_file | awk '{print $2}'`
        #
        while [[ "$events" -le "$number" ]]; do
            #
    	    $dprof -e $dprof_spec -d 0 ./$puse 2>$f_IP_exec_raw
        	#
            events=`wc -l $f_IP_exec_raw | awk '{print $1}'`
        done
        #
        awk '{print $1 " " $6}' $f_IP_exec_raw >$f_IP_exec
        #
        bash $dprof2ptrace $f_IP_exec >$f_ptrace
        #
        $ldis -L $puse >$f_stat
        #
        # ldis hack
        if file $f_stat | grep ISO-8859 &>/dev/null;then
            iconv -f koi8-r -t utf-8 -o $f_stat.u $f_stat
            mv $f_stat.u $f_stat
        fi
    fi

    #
    # Clone & Run
    #
    for core in $(get_order_proc);do
        #
        ((thread = core + 1))
        #
        f_thread="thread_${puse}_${thread}"
        f_time="$f_base_time.$thread"
        #
        cp $puse $f_thread
        #
        if test $el_util_f = 0;then
            /usr/bin/time --format=:%e:real:%U:user:%S:sys taskset -c $core $huge_run $run_spec ./$f_thread 2>$f_time &
        else
            ./proc_run 98F$core /usr/bin/time --format=:%e:real:%U:user:%S:sys ./proc_run 90F$core $huge_run $run_spec ./$f_thread 2>$f_time &
        fi
    done
    #
    wait $(jobs -p)

    # TS stop
    tunit_stop_point=$(date +"%d.%m.%YT%H:%M:%S")
    #

    # TS save
    echo "$tunit_start_point|$tunit_stop_point" > $f_ts_point

    #
    # AWK Program
    #
    [[ "$platform" == "e2k" ]] && result_program="/real/ {tim=\$2; if (timm<tim) {timm=tim}} /исполнено операций/ {op=\$2}  /литерала/ {lit=\$2} /плавающих операций/ {fp=\$2} /арифметико-логических/ {alu=\$2} END {print \"fp \" $num_cpu*fp/timm/$fp_spec \"opers \" $num_cpu*(op-lit)/timm/$oplit_spec \" ALU \" $num_cpu*alu/timm/$alu_spec}"
    #
    # sparc get operation count
    if echo $puse | grep 32 &>/dev/null;then
        op_cnt=$op_cnt_32
    else
        op_cnt=$op_cnt_64
    fi

    [[ "$platform" == "sparc64" ]] && result_program="/real/ {tim=\$2; if (timm<tim) {timm=tim}} END {print \"fp \" $op_cnt*$num_cpu/timm}"
    #
    [[ -z "$result_program" ]] && echo "Unknown platform: $platform" && exit 2

    #
    # Create result
    #
    awk -F':' "$result_program" $f_base_time.* $f_stat >$f_result

    #
done

# return huge
echo $nr_hugepages > /proc/sys/vm/nr_hugepages
echo $transparent_hugepage > /sys/kernel/mm/transparent_hugepage/enabled

#
# Print & save result
#
rm -f $result
touch $result
#
for res in $log_dir/*raw.result;do
    #
    fp=`cat ${res} | awk '{printf("%.2f", $2);}'`
    re='^[+-]?[0-9]+([.][0-9]+)?$'
    #
    if ! [[ $fp =~ $re ]] ; then
        echo "error: $fp is not number result" >&2
        exit 1
    fi
done

#
for res in $log_dir/*raw.result;do
    #
    test_name=$(basename $res | cut -d '.' -f1)
    #
    fp=`cat ${res} | awk '{printf("%.2f", $2);}'`
    oper=`cat ${res} | awk '{printf("%.2f", $4);}'`
    alu=`cat ${res} | awk '{printf("%.2f", $6);}'`
    #
    # Save result
    #
    ts_point=$(tail -1 $log_dir/$test_name*.ts.point)
    #
    echo "$test_name|$num_cpu|$fp|$ts_point" >> $result
    #
    # Print result
    #
    echo "${test_name} ${oper} ${alu} ${fp}"
done | awk ' \
BEGIN {
printf (" _________________________________ \n");
printf ("|                 |               |\n");
printf ("|      Тест       |     GFLOPs    |\n");
printf ("|_________________|_______________|\n");
     n=0;
     }
{
      test=$1
      oper=$2
      alu=$3
      fp=$4
    printf ("| %-15s |     %9.2f |\n", test, fp );
}
 END {
printf ("|_________________|_______________|\n");

}'

#
# Fin Clear
#
rm -rf thread_*
